﻿using System;
using System.Collections.Generic;

namespace FeatureToggles
{
    public class FeatureToggles
    {
        #region private fields

        /// <summary>
        /// Dictionary<String, Boolean>
        /// </summary>
        private static Dictionary<String, Boolean> _featureNamesDict = new Dictionary<String, Boolean>();

        /// <summary>
        /// SQLFeatureToggleStore
        /// </summary>
        private static SQLFeatureToggleStore _sQLFeatureToggleStore;

        #endregion private fields

        #region public methods
        
        /// <summary>
        /// RegisterIfNotExists
        /// </summary>
        /// <param name="pFeatureName"></param>
        public static void RegisterIfNotExists(String pFeatureName)
        {
            if (!_featureNamesDict.ContainsKey(pFeatureName))
            {
                Register(pFeatureName);
            }
        }    // end RegisterIfNotExists  

        /// <summary>
        /// Register
        /// </summary>
        /// <param name="pFeatureName"></param>
        public static void Register(String pFeatureName)
        {

            _featureNamesDict.Add(pFeatureName, false);
        }    //end Register

        /// <summary>
        /// Exists
        /// </summary>
        /// <param name="pFeatureName"></param>
        /// <returns></returns>
        public static Boolean Exists(String pFeatureName)
        {
            return _featureNamesDict.ContainsKey(pFeatureName);
        }   // end Exists

  
        /// <summary>
        /// SaveFeatureToggleState
        /// </summary>
        public static void SaveFeatureToggleState()
        {
            SaveFeatureToggleState(_featureNamesDict);
        }   // end SaveFeatureToggleState

        /// <summary>
        /// SaveFeatureToggleState
        /// </summary>
        /// <param name="pFeatureTogglesDict"></param>
        public static void SaveFeatureToggleState(Dictionary<String, Boolean> pFeatureTogglesDict)
   
        {
            try
            {
                FeatureToggles._sQLFeatureToggleStore.SaveFeaturesToDatabase(pFeatureTogglesDict);
            }

            catch (Exception ex)
            {
                throw new SaveToStoreException(ex);
            }

        }   //  end SaveFeatureToggleState


        /// <summary>
        /// Enable
        /// </summary>
        /// <param name="pFeatureName"></param>
        public static void Enable(String pFeatureName)
        {
            _featureNamesDict[pFeatureName] = true;   

        }    //end Enable      

        /// <summary>
        /// Disable
        /// </summary>
        /// <param name="pFeatureName"></param>
        public static void Disable(String pFeatureName)
        {
            _featureNamesDict[pFeatureName] = false;      
        }   //end Disable

        /// <summary>
        /// IsEnabled
        /// </summary>
        /// <param name="pFeatureName"></param>
        /// <returns></returns>
        public static Boolean IsEnabled(String pFeatureName)
        {
            if (_featureNamesDict.ContainsKey(pFeatureName))
            {
                return _featureNamesDict[pFeatureName];
             }
            
            {
                RegisterIfNotExists(pFeatureName);
                return false;
            }
        }   //end IsEnabled

        /// <summary>
        /// Initialize
        /// </summary>
        /// <param name="pSQLFeatureToggleStore"></param>
        public static void Initialize(SQLFeatureToggleStore pSQLFeatureToggleStore)
        {
            _featureNamesDict = pSQLFeatureToggleStore.LoadFeaturesFromDatabase();
            _sQLFeatureToggleStore = pSQLFeatureToggleStore;
        }    //Initialize 
        
        #endregion public methods

    }   //end class FeatureToggles
}   //end namespace FeatureToggles