﻿#region usings
using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
#endregion usings

#region MEF usings
using System.ComponentModel.Composition;            //MEF Libraries
using System.ComponentModel.Composition.Hosting;    //MEF Libraries
using AdvancedCalculator.MEF;                       //MEF Extension Classes
#endregion MEF usings

namespace AdvancedCalculator
{
    public partial class AdvancedCalculatorForm : Form
    {
        #region MEF ImportMany
        [ImportMany]
        IEnumerable<Lazy<ICalculationInterface, ICalculationOperation>> _ImportedOperations = null; // MEF
        #endregion MEF ImportMany

        #region private fields MEF
        private CompositionContainer _CompositionContainer;     //  MEF
        private AggregateCatalog _AggregateCatalog;             //  MEF
        private AssemblyCatalog _AssemblyCatalog;               //  MEF
        #endregion private fields MEF

        #region private fields
        private Stack _ValuesStack = new Stack();
        private Stack _MemoryStack = new Stack();
        private Decimal _ValueStored;
        private Decimal _DecResult = 0.0M;
        private Decimal _ValueEntered = 0.0M;
        private string _Operation;
        private string _PriorOperation;
        private Boolean _ClearEntry = false;
        #endregion private fields

        #region private fields for Feature Toggles
        private Boolean _IsKeyPadFeatureEnabled = false;
        private Boolean _IsKeyPadFeatureVisible = false;
        private Boolean _IsMemoryFeatureEnabled = false;
        private Boolean _IsMemoryFeatureVisible = false;
        private Boolean _IsAdvancedFeatureVisible = false;
        private Boolean _IsAdvancedFeatureEnabled = false;
        #endregion private fields for Feature Toggles

        #region public methods

        /// <summary>
        /// AdvancedCalculatorForm default constructor
        /// </summary>
        public AdvancedCalculatorForm()
        {
            InitializeComponent();
            rtbEntry.Focus();

            #region MEF Setup
            _AggregateCatalog = new AggregateCatalog();                             // MEF
            _AssemblyCatalog = new AssemblyCatalog(typeof(Program).Assembly);       // MEF
            _AggregateCatalog.Catalogs.Add(_AssemblyCatalog);                       // MEF
            _CompositionContainer = new CompositionContainer(_AggregateCatalog);    // MEF

            try
            {
                this._CompositionContainer.ComposeParts(this);
            }
            catch (CompositionException pCompositionException)
            {
                throw new Exception(pCompositionException.Message);
            }
            #endregion MEF Setup

        }   // end AdvancedCalculatorForm public constructor

        #endregion public methods

        #region private methods
        /// <summary>
        /// 
        /// </summary>
        /// <param name="valueIn"></param>
        /// <returns></returns>
        private decimal StringToDecimal(string valueIn)
        {
            decimal returnVal = 0;
            if (valueIn != String.Empty)
            {
                returnVal = Convert.ToDecimal(valueIn);
            }
            return returnVal;
        }   // StringToDecimal

        /// <summary>
        /// SetBasicVisible
        /// </summary>
        /// <param name="state"></param>
        private void SetStandardFeatureVisible(bool state)
        {
            this.rtbEntry.Visible = state;
            this.rtbEntry.Visible = state;
            this.btnSubtract.Visible = state;
            this.btnMultiply.Visible = state;
            this.btnDivide.Visible = state;
            this.btnAdd.Visible = state;
            this.btnEqual.Visible = state;
            this.lblVersion.Visible = state;
            this.lblRunning.Visible = state;
            this.lblEntry.Visible = state;
            this.rtbRunning.Visible = state;
            this.tbVersion.Visible = state;
            this.btnClearEntry.Visible = state;
            this.btnClear.Visible = state;

        }   // end SetStandardFeatureVisible

        /// <summary>
        /// SetBasicEnabled
        /// </summary>
        /// <param name="state"></param>
        private void SetStandardFeatureEnabled(bool state)
        {
            this.lblEntry.Enabled = state;
            this.lblRunning.Enabled = state;
            this.rtbEntry.Enabled = state;
            this.btnSubtract.Enabled = state;
            this.btnMultiply.Enabled = state;
            this.btnDivide.Enabled = state;
            this.btnAdd.Enabled = state;
            this.btnEqual.Enabled = state;
            this.rtbEntry.Enabled = state;
            this.btnClearEntry.Enabled = state;
            this.btnClear.Enabled = state;
        }   // SetStandardFeatureEnabled

        /// <summary>
        /// SetAdvancedVisible
        /// </summary>
        /// <param name="state"></param>
        private void SetAdvancedFeatureVisible(bool state)
        {
            this.btnBkspc.Visible = state;            
            this.btnSqrt.Visible = state;
            this.btnPercent.Visible = state;
            this.btnReciprocal.Visible = state;
            this.btnPlusMinus.Visible = state;
        }   // end SetAdvancedFeatureVisible

        /// <summary>
        /// SetAdvancedEnabled
        /// </summary>
        /// <param name="state"></param>
        private void SetAdvancedFeatureEnabled(bool state)
        {
            this.btnBkspc.Enabled = state;
            this.btnSqrt.Enabled = state;
            this.btnPercent.Enabled = state;
            this.btnReciprocal.Enabled = state;
            this.btnPlusMinus.Enabled = state;

        }   // end SetAdvancedFeatureEnabled

        /// <summary>
        /// SetKeyPadVisible
        /// </summary>
        /// <param name="state"></param>
        private void SetKeyPadFeatureVisible(bool state)
        {
            this.btn0.Visible = state;
            this.btn1.Visible = state;
            this.btn2.Visible = state;
            this.btn3.Visible = state;
            this.btn4.Visible = state;
            this.btn5.Visible = state;
            this.btn6.Visible = state;
            this.btn7.Visible = state;
            this.btn8.Visible = state;
            this.btn9.Visible = state;
            this.btnPoint.Visible = state;

        }   // end SetKeyPadFeatureVisible

        /// <summary>
        /// SetKeyPadEnabled
        /// </summary>
        /// <param name="state"></param>
        private void SetKeyPadFeatureEnabled(bool state)
        {
            this.btn0.Enabled = state;
            this.btn1.Enabled = state;
            this.btn2.Enabled = state;
            this.btn3.Enabled = state;
            this.btn4.Enabled = state;
            this.btn5.Enabled = state;
            this.btn6.Enabled = state;
            this.btn7.Enabled = state;
            this.btn8.Enabled = state;
            this.btn9.Enabled = state;
            this.btnPoint.Enabled = state;

        }   // end SetKeyPadFeatureEnabled

        /// <summary>
        /// SetMemoryVisible
        /// </summary>
        /// <param name="state"></param>
        private void SetMemoryFeatureVisible(bool state)
        {
            this.btnMem_Minus.Visible = state;
            this.btnMem_Clear.Visible = state;
            this.btnMem_Recall.Visible = state;
            this.btnMemoryAdd.Visible = state;
            this.btnMemSave.Visible = state;
        }   // SetMemoryFeatureVisible

        /// <summary>
        /// SetMemoryEnabled
        /// </summary>
        /// <param name="state"></param>
        private void SetMemoryFeatureEnabled(bool state)
        {
            this.btnMem_Minus.Enabled = state;
            this.btnMem_Clear.Enabled = state;
            this.btnMem_Recall.Enabled = state;
            this.btnMemoryAdd.Enabled = state;
            this.btnMemSave.Enabled = state;
        } // end SetMemoryFeatureEnabled

        private void ProcessButtonClick(object pSender, EventArgs pEventArgs)
        {
            if (!_ClearEntry)
            {
                _ValueEntered = StringToDecimal(rtbEntry.Text);
                Button myButton = (Button)pSender;
                _Operation = myButton.Text;
                if (_Operation == Constants.equalSign)
                {
                    _Operation = _PriorOperation;
                }
                else
                {
                    _PriorOperation = _Operation;
                }

                _ValuesStack.Push(_ValueEntered);

                if (_ValuesStack.Count > 1)
                {
                    _DecResult = ProcessCalculationExtension();
                    rtbEntry.Text = _DecResult.ToString();
                }
                else
                {
                    //rtbEntry.Text = String.Empty;
                }
                rtbRunning.Text = rtbRunning.Text + _ValueEntered.ToString() + Constants.litSpace +
                _Operation + Constants.litSpace;
            }
            //_ClearEntry = true;
            rtbEntry.Focus();
            rtbEntry.SelectAll();
        }   // end ProcessCalculation

        private Decimal ProcessCalculationExtension()
        {
            _ValueEntered = (Decimal)_ValuesStack.Pop();
            if (_ValuesStack.Count > 0)
            {
                _ValueStored = (Decimal)_ValuesStack.Pop();
            }
            _ValuesStack.Clear();

            {
                Boolean calculationFound = false;
                foreach (Lazy<AdvancedCalculator.MEF.ICalculationInterface, AdvancedCalculator.MEF.ICalculationOperation>
                    myOperation in _ImportedOperations)
                {
                    if (myOperation.Metadata.Operation.Equals(_Operation))
                    {
                        _DecResult = myOperation.Value.Calculation(_ValueStored, _ValueEntered);
                        calculationFound = true;
                        break;
                    }
                }
                if (!calculationFound)
                {
                    throw new NotImplementedException();
                }
            }
            _ValuesStack.Push(_DecResult);

            return _DecResult;
        }   //  end ProcessCalculationExtension
        private void AdvancedCalculator_Load(object sender, EventArgs e)
        {

            EvaluateFeatureToggles();

        }   // end AdvancedCalculator_Load

        /// <summary>
        /// EvaluateFeatureToggles
        /// </summary>
        private void EvaluateFeatureToggles()
        {
            this.tbVersion.Text = Properties.CommonSettings.Default.Version;
            if (Properties.CommonSettings.Default.DatabaseSettings)
            {
                GetFeatureTogglesFromStore();
            }
            else
            {
                GetFeatureTogglesFromSettings();
            }
            if (_IsMemoryFeatureEnabled)
            {
                _IsMemoryFeatureVisible = true;
            }
            if (_IsKeyPadFeatureEnabled)
            {
                _IsKeyPadFeatureVisible = true;
            }
            if (_IsAdvancedFeatureEnabled)
            {
                _IsAdvancedFeatureVisible = true;
            }
            SetKeyPadFeatureEnabled(_IsKeyPadFeatureEnabled);
            SetKeyPadFeatureVisible(_IsKeyPadFeatureVisible);
            SetMemoryFeatureEnabled(_IsMemoryFeatureEnabled);
            SetMemoryFeatureVisible(_IsMemoryFeatureVisible);
            SetAdvancedFeatureEnabled(_IsAdvancedFeatureEnabled);
            SetAdvancedFeatureVisible(_IsAdvancedFeatureVisible);
        }   // end EvaluateFeatureToggles

        private void GetFeatureTogglesFromSettings()
        {
            _IsKeyPadFeatureEnabled = Properties.ConfigSettings.Default.KeyPadFeatureEnabled;
            _IsKeyPadFeatureVisible = Properties.ConfigSettings.Default.KeyPadFeatureVisible;
            _IsMemoryFeatureEnabled = Properties.ConfigSettings.Default.MemoryFeatureEnabled;
            _IsMemoryFeatureVisible = Properties.ConfigSettings.Default.MemoryFeatureVisible;
            _IsAdvancedFeatureEnabled = Properties.ConfigSettings.Default.AdvancedFeatureEnabled;
            _IsAdvancedFeatureVisible = Properties.ConfigSettings.Default.AdvancedFeatureVisible;
            tbMode.Text = Constants.configSettings;
        }   // end GetFeatureTogglesFromSettings

        private void GetFeatureTogglesFromStore()
        {
            _IsKeyPadFeatureEnabled = FeatureToggles.FeatureToggles.IsEnabled(
                    Properties.FeatureToggleSettings.Default.KeyPadFeatureEnabled);
            _IsKeyPadFeatureVisible = FeatureToggles.FeatureToggles.IsEnabled(
                    Properties.FeatureToggleSettings.Default.KeyPadFeatureVisible);
            _IsMemoryFeatureEnabled = FeatureToggles.FeatureToggles.IsEnabled(
                   Properties.FeatureToggleSettings.Default.MemoryFeatureEnabled);
            _IsMemoryFeatureVisible = FeatureToggles.FeatureToggles.IsEnabled(
                    Properties.FeatureToggleSettings.Default.MemoryFeatureVisible);
            _IsAdvancedFeatureEnabled = FeatureToggles.FeatureToggles.IsEnabled(
                    Properties.FeatureToggleSettings.Default.AdvancedFeatureEnabled);
            _IsAdvancedFeatureVisible = FeatureToggles.FeatureToggles.IsEnabled(
                    Properties.FeatureToggleSettings.Default.AdvancedFeatureVisible);

            tbMode.Text = Constants.databaseSettings;

        }   // end GetFeatureFlagsFromStore
        private void ProcessKeyPad(object sender, EventArgs e)
        {
            Button myButton = (Button)sender;
            char[] charArray = myButton.Text.ToCharArray();
            char digit = charArray[0];
            if (_ClearEntry)
            {
                rtbEntry.Text = String.Empty;
            }
            _ClearEntry = false;
            this.rtbEntry.Text = this.rtbEntry.Text += digit.ToString();
        }   // end ProcessKeyPad

        private string Truncate(string pSource, int pLength)
        {
            if (pLength > 0)
            {
                if (pSource.Length > pLength)
                {
                    pSource = pSource.Substring(0, pLength);
                }
            }
            else
            {
                pSource = string.Empty;
            }
            return pSource;
        }

        #endregion private methods

        #region User Interface events

        #region Standard functions feature events

        /// <summary>
        /// btnAdd_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnAdd_Click(object sender, EventArgs e)
        {
            ProcessButtonClick(sender, e); ;
        }   // end btnAdd_Click

        /// <summary>
        /// btnSubtract_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSubtract_Click(object sender, EventArgs e)
        {
            ProcessButtonClick(sender, e); ;
        }   // end btnSubtract_Click

        /// <summary>
        /// btnMultiply_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMultiply_Click(object sender, System.EventArgs e)
        {
            ProcessButtonClick(sender, e);

        }   // end btnMultiply_Click

        /// <summary>
        /// btnDivide_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDivide_Click(object sender, EventArgs e)
        {

            ProcessButtonClick(sender, e);

        }   // end btnDivide_Click

        /// <summary>
        /// btnClearEntry_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClearEntry_Click(object sender, EventArgs e)
        {
            this.rtbEntry.Text = String.Empty;
            rtbEntry.Focus();
        }   //  btnClearEntry_Click

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnClear_Click(object sender, EventArgs e)
        {
            this.rtbEntry.Text = String.Empty;
            this.rtbRunning.Text = String.Empty;
            _ValuesStack.Clear();
            rtbEntry.Focus();
        }   // end  btnClear_Click

        #endregion standard functions feature events

        #region Memory functions feature events

        /// <summary>
        /// btnMemoryAdd_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMemoryAdd_Click(object sender, EventArgs e)
        {
            _ValueStored = 0.0M;
            _ValueEntered = StringToDecimal(rtbEntry.Text);
            if (_MemoryStack.Count > 0)
            {
                _ValueStored = (Decimal)_MemoryStack.Pop();
            }
            _MemoryStack.Push(_ValueStored);
            _MemoryStack.Push(_ValueStored + _ValueEntered);

        }       // end btnMemoryAdd_Click

        /// <summary>
        /// btnMemorySubtract_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMemorySubtract_Click(object sender, EventArgs e)
        {
            _ValueStored = 0.0M;
            _ValueEntered = StringToDecimal(rtbEntry.Text);
            if (_MemoryStack.Count > 0)
            {
                _ValueStored = (Decimal)_MemoryStack.Pop();
            }
            _MemoryStack.Push(_ValueStored);
            _MemoryStack.Push(_ValueStored - _ValueEntered);
            rtbEntry.Text = _ValueStored.ToString();

        } // end btnMemorySubtract_Click

        /// <summary>
        /// btnMem_Recall_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMem_Recall_Click(object sender, EventArgs e)
        {
            _ValueStored = 0.0M;
            _ValueEntered = StringToDecimal(rtbEntry.Text);
            if (_MemoryStack.Count > 0)
            {
                _ValueStored = (Decimal)_MemoryStack.Pop();
            }
            _MemoryStack.Push(_ValueStored);
            rtbEntry.Text = _ValueStored.ToString();

        }   //  btnMem_Recall_Click

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMemSave_Click(object sender, EventArgs e)
        {
            _ValueEntered = StringToDecimal(rtbEntry.Text);
            _MemoryStack.Push(_ValueEntered);

        }   //  btnMemSave_Click

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMem_Clear_Click(object sender, EventArgs e)
        {
            _MemoryStack.Clear();

        }   //  btnMem_Clear_Click

        #endregion Memory functions feature events

        #region keypad feture events
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPoint_Click(object sender, EventArgs e)
        {
            ProcessKeyPad(sender, e);
        }   //  btnPoint_Click

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn0_Click(object sender, EventArgs e)
        {
            ProcessKeyPad(sender, e);
        }   //

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn1_Click(object sender, EventArgs e)
        {
            ProcessKeyPad(sender, e);
        }   //

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn2_Click(object sender, EventArgs e)
        {
            ProcessKeyPad(sender, e);
        }   //

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn3_Click(object sender, EventArgs e)
        {
            ProcessKeyPad(sender, e);

        }   //

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn4_Click(object sender, EventArgs e)
        {
            ProcessKeyPad(sender, e);

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn5_Click(object sender, EventArgs e)
        {
            ProcessKeyPad(sender, e);
        }   //

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>       
        private void btn6_Click(object sender, EventArgs e)
        {
            ProcessKeyPad(sender, e);
        }   //

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn7_Click(object sender, EventArgs e)
        {
            ProcessKeyPad(sender, e);

        }   //

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn8_Click(object sender, EventArgs e)
        {
            ProcessKeyPad(sender, e);

        }   //

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn9_Click(object sender, EventArgs e)
        {
            ProcessKeyPad(sender, e);
        }


        #endregion keypad feture events

        #region advanced functions feature events

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEqual_Click(object sender, EventArgs e)
        {

            ProcessButtonClick(sender, e);
            _ClearEntry = true;
            rtbRunning.Text = String.Empty;
            rtbEntry.Focus();

        }   // end btnEqual_Click

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPlusMinus_Click(object sender, EventArgs e)
        {
            _ValuesStack.Push(0.0M);
            ProcessButtonClick(sender, e);

        }   //  end btnPlusMinus_Click

        /// <summary>
        /// btnSqrt_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSqrt_Click(object sender, EventArgs e)
        {
            ProcessButtonClick(sender, e);

        }   // btnSqrt_Click

        /// <summary>
        /// btnPercent_Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPercent_Click(object sender, EventArgs e)
        {
            ProcessButtonClick(sender, e);
        }   //  btnPercent_Click

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSqrt_Click_1(object sender, EventArgs e)
        {
            _ValuesStack.Push(0.0M);
            ProcessButtonClick(sender, e);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnReciprocal_Click(object sender, EventArgs e)
        {
            if (_ValuesStack.Count < 1)
            {
                _ValuesStack.Push(0.0M);
            }
            ProcessButtonClick(sender, e);
        }

        private void btnBkspc_Click(object sender, EventArgs e)
        {

            rtbEntry.Text = Truncate(rtbEntry.Text, rtbEntry.Text.Length - 1);

        }   // end btnBkspc_Click

        private void btnMem_Minus_Click(object sender, EventArgs e)
        {
            _ValueEntered = StringToDecimal(rtbEntry.Text);
            _ValueStored = (Decimal)_MemoryStack.Pop();
            _MemoryStack.Push(_ValueStored);
            _DecResult = _ValueStored - _ValueEntered;
            _MemoryStack.Push(_DecResult);
        }
        #endregion advanced functions feature events

        #endregion User Interface events

    }   // end class AdvancedCalculatorForm
}   // end namespace AdvancedCalculator
